summary refs log tree commit diff
path: root/crypto/src/asn1/DerBMPString.cs
diff options
context:
space:
mode:
Diffstat (limited to 'crypto/src/asn1/DerBMPString.cs')
-rw-r--r--crypto/src/asn1/DerBMPString.cs108
1 files changed, 67 insertions, 41 deletions
diff --git a/crypto/src/asn1/DerBMPString.cs b/crypto/src/asn1/DerBMPString.cs
index 4e35dfff8..a289eed1b 100644
--- a/crypto/src/asn1/DerBMPString.cs
+++ b/crypto/src/asn1/DerBMPString.cs
@@ -1,4 +1,5 @@
 using System;
+using System.IO;
 
 using Org.BouncyCastle.Utilities;
 
@@ -10,7 +11,17 @@ namespace Org.BouncyCastle.Asn1
     public class DerBmpString
 		: DerStringBase
     {
-        private readonly string str;
+        internal class Meta : Asn1UniversalType
+        {
+            internal static readonly Asn1UniversalType Instance = new Meta();
+
+            private Meta() : base(typeof(DerBmpString), Asn1Tags.BmpString) {}
+
+            internal override Asn1Object FromImplicitPrimitive(DerOctetString octetString)
+            {
+                return CreatePrimitive(octetString.GetOctets());
+            }
+        }
 
 		/**
          * return a BMP string from the given object.
@@ -18,13 +29,29 @@ namespace Org.BouncyCastle.Asn1
          * @param obj the object we want converted.
          * @exception ArgumentException if the object cannot be converted.
          */
-        public static DerBmpString GetInstance(
-            object obj)
+        public static DerBmpString GetInstance(object obj)
         {
             if (obj == null || obj is DerBmpString)
             {
                 return (DerBmpString)obj;
             }
+            else if (obj is IAsn1Convertible)
+            {
+                Asn1Object asn1Object = ((IAsn1Convertible)obj).ToAsn1Object();
+                if (asn1Object is DerBmpString)
+                    return (DerBmpString)asn1Object;
+            }
+            else if (obj is byte[])
+            {
+                try
+                {
+                    return (DerBmpString)Meta.Instance.FromByteArray((byte[])obj);
+                }
+                catch (IOException e)
+                {
+                    throw new ArgumentException("failed to construct BMP string from byte[]: " + e.Message);
+                }
+            }
 
             throw new ArgumentException("illegal object in GetInstance: " + Platform.GetTypeName(obj));
         }
@@ -32,48 +59,35 @@ namespace Org.BouncyCastle.Asn1
         /**
          * return a BMP string from a tagged object.
          *
          * @param obj the tagged object holding the object we want
          * @param explicitly true if the object is meant to be explicitly
          *              tagged false otherwise.
          * @exception ArgumentException if the tagged object cannot
          *              be converted.
          */
-        public static DerBmpString GetInstance(
-            Asn1TaggedObject	obj,
-            bool				isExplicit)
+        public static DerBmpString GetInstance(Asn1TaggedObject taggedObject, bool declaredExplicit)
         {
-			Asn1Object o = obj.GetObject();
-
-			if (isExplicit || o is DerBmpString)
-			{
-				return GetInstance(o);
-			}
-
-			return new DerBmpString(Asn1OctetString.GetInstance(o).GetOctets());
+            return (DerBmpString)Meta.Instance.GetContextInstance(taggedObject, declaredExplicit);
         }
 
-		/**
-         * basic constructor - byte encoded string.
-         */
-        [Obsolete("Will become internal")]
-        public DerBmpString(byte[] str)
+        private readonly string m_str;
+
+        internal DerBmpString(byte[] contents)
         {
-			if (str == null)
-				throw new ArgumentNullException("str");
+			if (null == contents)
+				throw new ArgumentNullException("contents");
 
-            int byteLen = str.Length;
+            int byteLen = contents.Length;
             if (0 != (byteLen & 1))
-                throw new ArgumentException("malformed BMPString encoding encountered", "str");
+                throw new ArgumentException("malformed BMPString encoding encountered", "contents");
 
             int charLen = byteLen / 2;
             char[] cs = new char[charLen];
 
             for (int i = 0; i != charLen; i++)
             {
-                cs[i] = (char)((str[2 * i] << 8) | (str[2 * i + 1] & 0xff));
+                cs[i] = (char)((contents[2 * i] << 8) | (contents[2 * i + 1] & 0xff));
             }
 
-            this.str = new string(cs);
+            m_str = new string(cs);
         }
 
         internal DerBmpString(char[] str)
@@ -81,7 +95,7 @@ namespace Org.BouncyCastle.Asn1
             if (str == null)
                 throw new ArgumentNullException("str");
 
-            this.str = new string(str);
+            m_str = new string(str);
         }
 
         /**
@@ -92,23 +106,24 @@ namespace Org.BouncyCastle.Asn1
 			if (str == null)
 				throw new ArgumentNullException("str");
 
-            this.str = str;
+            m_str = str;
         }
 
         public override string GetString()
         {
-            return str;
+            return m_str;
         }
 
-		protected override bool Asn1Equals(
-			Asn1Object asn1Object)
+        protected override bool Asn1Equals(Asn1Object asn1Object)
         {
-			DerBmpString other = asn1Object as DerBmpString;
-
-			if (other == null)
-				return false;
+            DerBmpString that = asn1Object as DerBmpString;
+            return null != that
+                && this.m_str.Equals(that.m_str);
+        }
 
-			return this.str.Equals(other.str);
+        protected override int Asn1GetHashCode()
+        {
+            return m_str.GetHashCode();
         }
 
         internal override IAsn1Encoding GetEncoding(int encoding)
@@ -123,7 +138,7 @@ namespace Org.BouncyCastle.Asn1
 
         private byte[] GetContents()
         {
-            char[] c = str.ToCharArray();
+            char[] c = m_str.ToCharArray();
             byte[] b = new byte[c.Length * 2];
 
             for (int i = 0; i != c.Length; i++)
@@ -134,5 +149,16 @@ namespace Org.BouncyCastle.Asn1
 
             return b;
         }
+
+        internal static DerBmpString CreatePrimitive(byte[] contents)
+        {
+            return new DerBmpString(contents);
+        }
+
+        internal static DerBmpString CreatePrimitive(char[] str)
+        {
+            // TODO[asn1] Asn1InputStream has a validator/converter that should be unified in this class somehow
+            return new DerBmpString(str);
+        }
     }
 }
@@ -32,48 +59,35 @@ namespace Org.BouncyCastle.Asn1
          * return a BMP string from a tagged object.
          *
-         * @param obj the tagged object holding the object we want
-         * @param explicitly true if the object is meant to be explicitly
-         *              tagged false otherwise.
-         * @exception ArgumentException if the tagged object cannot
-         *              be converted.
+         * @param taggedObject the tagged object holding the object we want
+         * @param declaredExplicit true if the object is meant to be explicitly tagged false otherwise.
+         * @exception ArgumentException if the tagged object cannot be converted.
          */